@@ -70,9 +70,10 @@ def data_augmentation_dataset(
         if as_tokens:
             with open(file_path) as json_file:
                 file = json.load(json_file)
-                ids, programs = file["ids"], file["programs"]
+                ids = file["ids"]
+                programs = file["programs"] if "programs" in file else None
 
-            if tokenizer.unique_track:
+            if tokenizer.one_token_stream:
                 ids = [ids]
 
             # Perform data augmentation for each track
@@ -89,31 +90,44 @@ def data_augmentation_dataset(
             augmented_tokens: Dict[
                 Tuple[int, int, int], List[Union[int, List[int]]]
             ] = {}
-            for track, (_, is_drum) in zip(ids, programs):
-                if is_drum:  # we dont augment drums
+            for ti, track in enumerate(ids):
+                # we dont augment drums
+                if programs is not None and programs[ti][1]:  # drums
                     continue
+                elif (
+                    tokenizer.one_token_stream
+                    and programs is not None
+                    and all(p[1] for p in programs)
+                ):
+                    continue
+                corrected_offsets = deepcopy(offsets)
+                vel_dim = int(128 / len(tokenizer.velocities))
+                corrected_offsets[1] = [
+                    int(off / vel_dim) for off in corrected_offsets[1]
+                ]
                 aug = data_augmentation_tokens(
                     np.array(track),
                     tokenizer,
-                    *offsets,
+                    *corrected_offsets,
                     all_offset_combinations=all_offset_combinations,
                 )
                 if len(aug) == 0:
                     continue
                 for aug_offsets, seq in aug:
-                    if tokenizer.unique_track:
+                    if tokenizer.one_token_stream:
                         augmented_tokens[aug_offsets] = seq
                         continue
                     try:
                         augmented_tokens[aug_offsets].append(seq)
                     except KeyError:
                         augmented_tokens[aug_offsets] = [seq]
-            for i, (track, (_, is_drum)) in enumerate(
-                zip(ids, programs)
-            ):  # adding drums to all already augmented
-                if is_drum:
-                    for aug_offsets in augmented_tokens:
-                        augmented_tokens[aug_offsets].insert(i, track)
+            if not tokenizer.one_token_stream and programs is not None:
+                for i, (track, (_, is_drum)) in enumerate(
+                    zip(ids, programs)
+                ):  # adding drums to all already augmented
+                    if is_drum:
+                        for aug_offsets in augmented_tokens:
+                            augmented_tokens[aug_offsets].insert(i, track)
 
             # Save augmented tracks as json
             for aug_offsets, tracks_seq in augmented_tokens.items():
@@ -133,6 +147,8 @@ def data_augmentation_dataset(
                 nb_augmentations += 1
                 nb_tracks_augmented += len(tracks_seq)
             if copy_original_in_new_location and out_path is not None:
+                if tokenizer.one_token_stream:
+                    ids = ids[0]
                 tokenizer.save_tokens(
                     ids, out_path / f"{file_path.stem}.json", programs
                 )
@@ -220,7 +236,7 @@ def get_offsets(
             as a tuple of two booleans. (default: (True, True))
     :param midi: midi object to augment (default: None)
     :param ids: token ids as a list of tracks (default: None)
-    :return: augmented MIDI objects.
+    :return: the offsets of pitch, velocity and duration features, in "absolute" value
     """
     offsets = []
 
@@ -265,10 +281,11 @@ def get_offsets(
                     pitch_voc_idx, int(np.min(np.concatenate(ids_pitch)))
                 ].split("_")[1]
         offset_up = min(
-            nb_octave_offset, (tokenizer.pitch_range.stop - 1 - int(max_pitch)) // 12
+            nb_octave_offset,
+            (tokenizer.config.pitch_range[1] - 1 - int(max_pitch)) // 12,
         )
         offset_down = min(
-            nb_octave_offset, (int(min_pitch) - tokenizer.pitch_range.start) // 12
+            nb_octave_offset, (int(min_pitch) - tokenizer.config.pitch_range[0]) // 12
         )
 
         off = []
@@ -442,6 +459,17 @@ def data_augmentation_tokens(
             )
             note_off_tokens = np.array(tokenizer.token_ids_of_type("NoteOff"))
             mask_pitch = np.isin(tokens, pitch_tokens)
+            # If applicable, removes drum notes from the mask
+            if tokenizer.one_token_stream:
+                for idx, is_note in enumerate(mask_pitch):
+                    if (
+                        is_note
+                        and idx > 0
+                        and tokenizer[tokens[idx - 1]] == "Program_-1"
+                    ):
+                        mask_pitch[idx] = False
+                        if len(note_off_tokens) > 0:
+                            note_off_tokens[idx] = False
         else:
             pitch_tokens = np.array(tokenizer.token_ids_of_type("Pitch", pitch_voc_idx))
             mask_pitch = np.full_like(tokens, 0, dtype=np.bool_)
