@@ -1,9 +1,19 @@
 from .remi import REMI
-from .remi_plus import REMIPlus
 from .structured import Structured
 from .midi_like import MIDILike
 from .tsd import TSD
 from .cp_word import CPWord
 from .mumidi import MuMIDI
 from .octuple import Octuple
-from .octuple_mono import OctupleMono
+from .mmm import MMM
+
+__all__ = [
+    "MIDILike",
+    "REMI",
+    "TSD",
+    "Structured",
+    "Octuple",
+    "CPWord",
+    "MuMIDI",
+    "MMM",
+]
