@@ -1,5 +1,5 @@
 from math import ceil
-from pathlib import Path, PurePath
+from pathlib import Path
 from typing import List, Tuple, Dict, Optional, Union, Any
 
 import numpy as np
@@ -9,11 +9,6 @@ from ..midi_tokenizer import MIDITokenizer, _in_as_seq, _out_as_complete_seq
 from ..classes import TokSequence, Event
 from ..utils import detect_chords
 from ..constants import (
-    PITCH_RANGE,
-    NB_VELOCITIES,
-    BEAT_RES,
-    ADDITIONAL_TOKENS,
-    SPECIAL_TOKENS,
     TIME_DIVISION,
     TEMPO,
     MIDI_INSTRUMENTS,
@@ -42,42 +37,28 @@ class MuMIDI(MIDITokenizer):
     For generation, the decoding implies sample from several distributions, which can be
     very delicate. Hence, we do not recommend this tokenization for generation with small models.
 
+    **Add a `drum_pitch_range` entry in the config, mapping to a tuple of values to restrict the range of drum pitches
+    to use.**
+
     **Notes:**
         * Tokens are first sorted by time, then track, then pitch values.
         * Tracks with the same *Program* will be merged.
-
-    :param pitch_range: range of MIDI pitches to use
-    :param beat_res: beat resolutions, as a dictionary:
-            {(beat_x1, beat_x2): beat_res_1, (beat_x2, beat_x3): beat_res_2, ...}
-            The keys are tuples indicating a range of beats, ex 0 to 3 for the first bar, and
-            the values are the resolution to apply to the ranges, in samples per beat, ex 8
-    :param nb_velocities: number of velocity bins
-    :param additional_tokens: additional tokens (chords, time signature, rests, tempo...) to use,
-            to be given as a dictionary. (default: None is used)
-    :param special_tokens: list of special tokens. This must be given as a list of strings given
-            only the names of the tokens. (default: ``["PAD", "BOS", "EOS", "MASK"]``)
-    :param params: path to a tokenizer config file. This will override other arguments and
-            load the tokenizer based on the config file. This is particularly useful if the
-            tokenizer learned Byte Pair Encoding. (default: None)
-    :param drum_pitch_range: range of used MIDI pitches for drums exclusively
     """
 
-    def __init__(
-        self,
-        pitch_range: range = PITCH_RANGE,
-        beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
-        nb_velocities: int = NB_VELOCITIES,
-        additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-        special_tokens: List[str] = SPECIAL_TOKENS,
-        params: Union[str, Path] = None,
-        drum_pitch_range: range = DRUM_PITCH_RANGE,
-    ):
-        additional_tokens["Rest"] = False
-        additional_tokens["TimeSignature"] = False  # not compatible
-        self.drum_pitch_range = drum_pitch_range
-        self.programs = additional_tokens.get("programs", list(range(-1, 128)))
-        # used in place of positional encoding
-        self.max_bar_embedding = 60  # this attribute might increase during encoding
+    def _tweak_config_before_creating_voc(self):
+        self.config.use_rests = False
+        self.config.use_time_signatures = False
+        self.config.use_sustain_pedals = False
+        self.config.use_pitch_bends = False
+        self.config.use_programs = True
+        self.one_token_stream = True
+
+        if "drum_pitch_range" not in self.config.additional_params:
+            self.config.additional_params["drum_pitch_range"] = DRUM_PITCH_RANGE
+        if "max_bar_embedding" not in self.config.additional_params:
+            # this attribute might increase over tokenizations, if the tokenizer encounter longer MIDIs
+            self.config.additional_params["max_bar_embedding"] = 60
+
         self.vocab_types_idx = {
             "Pitch": 0,
             "DrumPitch": 0,
@@ -89,54 +70,12 @@ class MuMIDI(MIDITokenizer):
             "Velocity": -2,
             "Duration": -1,
         }
-        if additional_tokens["Chord"]:
+        if self.config.use_chords:
             self.vocab_types_idx["Chord"] = 0
-        if additional_tokens["Rest"]:
+        if self.config.use_rests:
             self.vocab_types_idx["Rest"] = 0
-        if additional_tokens["Tempo"]:
+        if self.config.use_tempos:
             self.vocab_types_idx["Tempo"] = -3
-        super().__init__(
-            pitch_range,
-            beat_res,
-            nb_velocities,
-            additional_tokens,
-            special_tokens,
-            True,
-            params=params,
-        )
-
-    def save_params(
-        self, out_path: Union[str, Path, PurePath], additional_attributes: Dict = None
-    ):
-        r"""Saves the config / parameters of the tokenizer in a json encoded file.
-        This can be useful to keep track of how a dataset has been tokenized.
-        **Note:** if you override this method, you should probably call it (super()) at the end
-            and use the additional_attributes argument.
-
-        :param out_path: output path to save the file.
-        :param additional_attributes: any additional information to store in the config file.
-                It can be used to override the default attributes saved in the parent method. (default: None)
-        """
-        if additional_attributes is None:
-            additional_attributes = {}
-        additional_attributes_tmp = {
-            "max_bar_embedding": self.max_bar_embedding,
-            "programs": self.programs,
-            "drum_pitch_range": (
-                self.drum_pitch_range.start,
-                self.drum_pitch_range.stop,
-            ),
-            **additional_attributes,
-        }
-        super().save_params(out_path, additional_attributes_tmp)
-
-    def load_params(self, config_file_path: Union[str, Path, PurePath]):
-        r"""Load the parameters of the tokenizer from a config file.
-
-        :param config_file_path: path to the tokenizer config file (encoded as json).
-        """
-        super().load_params(config_file_path)
-        self.drum_pitch_range = range(*self.drum_pitch_range)
 
     @_out_as_complete_seq
     def _midi_to_tokens(self, midi: MidiFile, *args, **kwargs) -> TokSequence:
@@ -152,51 +91,24 @@ class MuMIDI(MIDITokenizer):
         :param midi: the MIDI object to convert
         :return: sequences of tokens
         """
-        # Check if the durations values have been calculated before for this time division
-        if midi.ticks_per_beat not in self._durations_ticks:
-            self._durations_ticks[midi.ticks_per_beat] = np.array(
-                [
-                    (beat * res + pos) * midi.ticks_per_beat // res
-                    for beat, pos, res in self.durations
-                ]
-            )
-        # Preprocess the MIDI file
-        self.preprocess_midi(midi)
-
-        # Register MIDI metadata
-        self.current_midi_metadata = {
-            "time_division": midi.ticks_per_beat,
-            "tempo_changes": midi.tempo_changes,
-            "time_sig_changes": midi.time_signature_changes,
-            "key_sig_changes": midi.key_signature_changes,
-        }
-
-        # **************** OVERRIDE FROM HERE, KEEP THE LINES ABOVE IN YOUR METHOD ****************
-
         # Check bar embedding limit, update if needed
         nb_bars = ceil(midi.max_tick / (midi.ticks_per_beat * 4))
-        if self.max_bar_embedding < nb_bars:
-            for i in range(self.max_bar_embedding, nb_bars):
+        if self.config.additional_params["max_bar_embedding"] < nb_bars:
+            for i in range(self.config.additional_params["max_bar_embedding"], nb_bars):
                 self.add_to_vocab(f"BarPosEnc_{i}", 1)
-            self.max_bar_embedding = nb_bars
+            self.config.additional_params["max_bar_embedding"] = nb_bars
 
         # Convert each track to tokens (except first pos to track time)
         note_tokens = []
         for track in midi.instruments:
-            if track.program in self.programs:
-                note_tokens += self.track_to_tokens(track)
+            if track.program in self.config.programs:
+                note_tokens += self._track_to_tokens(track)
 
         note_tokens.sort(
             key=lambda x: (x[0].time, x[0].desc)
         )  # Sort by time then track
 
-        """from copy import deepcopy  # TODO remove
-        toto = deepcopy(note_tokens)
-        for i in range(len(toto)):
-            for j in range(1, len(toto[i])):
-                toto[i][j] = self[j, toto[i][j]]"""
-
-        ticks_per_sample = midi.ticks_per_beat / max(self.beat_res.values())
+        ticks_per_sample = midi.ticks_per_beat / max(self.config.beat_res.values())
         ticks_per_bar = midi.ticks_per_beat * 4
         tokens = []
 
@@ -205,18 +117,18 @@ class MuMIDI(MIDITokenizer):
         current_pos = -1
         current_track = -2  # because -2 doesn't exist
         current_tempo_idx = 0
-        current_tempo = self.current_midi_metadata["tempo_changes"][
+        current_tempo = self._current_midi_metadata["tempo_changes"][
             current_tempo_idx
         ].tempo
         for note_token in note_tokens:
             # (Tempo) update tempo values current_tempo
-            if self.additional_tokens["Tempo"]:
+            if self.config.use_tempos:
                 # If the current tempo is not the last one
                 if current_tempo_idx + 1 < len(
-                    self.current_midi_metadata["tempo_changes"]
+                    self._current_midi_metadata["tempo_changes"]
                 ):
                     # Will loop over incoming tempo changes
-                    for tempo_change in self.current_midi_metadata["tempo_changes"][
+                    for tempo_change in self._current_midi_metadata["tempo_changes"][
                         current_tempo_idx + 1 :
                     ]:
                         # If this tempo change happened before the current moment
@@ -242,7 +154,7 @@ class MuMIDI(MIDITokenizer):
                             f"BarPosEnc_{current_bar + i + 1}",
                             "PositionPosEnc_None",
                         ]
-                        if self.additional_tokens["Tempo"]:
+                        if self.config.use_tempos:
                             bar_token.append(f"Tempo_{current_tempo}")
                         tokens.append(bar_token)
                     current_bar += nb_new_bars
@@ -252,7 +164,7 @@ class MuMIDI(MIDITokenizer):
                     f"BarPosEnc_{current_bar}",
                     f"PositionPosEnc_{current_pos}",
                 ]
-                if self.additional_tokens["Tempo"]:
+                if self.config.use_tempos:
                     pos_token.append(f"Tempo_{current_tempo}")
                 tokens.append(pos_token)
             # Program (track)
@@ -263,7 +175,7 @@ class MuMIDI(MIDITokenizer):
                     f"BarPosEnc_{current_bar}",
                     f"PositionPosEnc_{current_pos}",
                 ]
-                if self.additional_tokens["Tempo"]:
+                if self.config.use_tempos:
                     track_token.append(f"Tempo_{current_tempo}")
                 tokens.append(track_token)
 
@@ -271,13 +183,13 @@ class MuMIDI(MIDITokenizer):
             note_token[0] = str(note_token[0])
             note_token.insert(1, f"BarPosEnc_{current_bar}")
             note_token.insert(2, f"PositionPosEnc_{current_pos}")
-            if self.additional_tokens["Tempo"]:
+            if self.config.use_tempos:
                 note_token.insert(3, f"Tempo_{current_tempo}")
             tokens.append(note_token)
 
         return TokSequence(tokens=tokens)
 
-    def track_to_tokens(self, track: Instrument) -> List[List[Union[Event, str]]]:
+    def _track_to_tokens(self, track: Instrument) -> List[List[Union[Event, str]]]:
         r"""Converts a track (miditoolkit.Instrument object) into a sequence of tokens (:class:`miditok.TokSequence`).
         For each note, it creates a time step as a list of tokens where (list index: token type):
         * 0: Pitch (as an Event object for sorting purpose afterwards)
@@ -289,7 +201,7 @@ class MuMIDI(MIDITokenizer):
         """
         # Make sure the notes are sorted first by their onset (start) times, second by pitch
         # notes.sort(key=lambda x: (x.start, x.pitch))  # done in midi_to_tokens
-        dur_bins = self._durations_ticks[self.current_midi_metadata["time_division"]]
+        dur_bins = self._durations_ticks[self._current_midi_metadata["time_division"]]
 
         tokens = []
         for note in track.notes:
@@ -324,14 +236,14 @@ class MuMIDI(MIDITokenizer):
                 )
 
         # Adds chord tokens if specified
-        if self.additional_tokens["Chord"] and not track.is_drum:
+        if self.config.use_chords and not track.is_drum:
             chords = detect_chords(
                 track.notes,
                 self._current_midi_metadata["time_division"],
-                chord_maps=self.additional_tokens["chord_maps"],
-                specify_root_note=self.additional_tokens["chord_tokens_with_root_note"],
+                chord_maps=self.config.chord_maps,
+                specify_root_note=self.config.chord_tokens_with_root_note,
                 beat_res=self._first_beat_res,
-                unknown_chords_nb_notes_range=self.additional_tokens["chord_unknown"],
+                unknown_chords_nb_notes_range=self.config.chord_unknown,
             )
             unsqueezed = []
             for c in range(len(chords)):
@@ -373,12 +285,18 @@ class MuMIDI(MIDITokenizer):
         :return: the midi object (miditoolkit.MidiFile)
         """
         assert (
-            time_division % max(self.beat_res.values()) == 0
-        ), f"Invalid time division, please give one divisible by {max(self.beat_res.values())}"
+            time_division % max(self.config.beat_res.values()) == 0
+        ), f"Invalid time division, please give one divisible by {max(self.config.beat_res.values())}"
         midi = MidiFile(ticks_per_beat=time_division)
-        midi.tempo_changes.append(TempoChange(TEMPO, 0))
-        ticks_per_sample = time_division // max(self.beat_res.values())
 
+        # Tempos
+        if self.config.use_tempos:
+            first_tempo = float(tokens.tokens[0][3].split("_")[1])
+        else:
+            first_tempo = TEMPO
+        midi.tempo_changes.append(TempoChange(first_tempo, 0))
+
+        ticks_per_sample = time_division // max(self.config.beat_res.values())
         tracks = {}
         current_tick = 0
         current_bar = -1
@@ -414,6 +332,12 @@ class MuMIDI(MIDITokenizer):
                     Note(vel, pitch, current_tick, current_tick + duration)
                 )
 
+            # Decode tempo if required
+            if self.config.use_tempos:
+                tempo_val = float(time_step[3].split("_")[1])
+                if tempo_val != midi.tempo_changes[-1].tempo:
+                    midi.tempo_changes.append(TempoChange(tempo_val, current_tick))
+
         # Appends created notes to MIDI object
         for program, notes in tracks.items():
             if int(program) == -1:
@@ -432,9 +356,9 @@ class MuMIDI(MIDITokenizer):
             midi.dump(output_path)
         return midi
 
-    def tokens_to_track(
+    def _tokens_to_track(
         self,
-        tokens: Union[TokSequence, List, np.ndarray, Any],
+        tokens: TokSequence,
         time_division: Optional[int] = TIME_DIVISION,
         program: Optional[Tuple[int, bool]] = (0, False),
     ):
@@ -448,7 +372,7 @@ class MuMIDI(MIDITokenizer):
         """
         pass
 
-    def _create_base_vocabulary(self, sos_eos_tokens: bool = None) -> List[List[str]]:
+    def _create_base_vocabulary(self) -> List[List[str]]:
         r"""Creates the vocabulary, as a list of string tokens.
         Each token as to be given as the form of "Type_Value", separated with an underscore.
         Example: Pitch_58
@@ -470,15 +394,24 @@ class MuMIDI(MIDITokenizer):
         vocab = [[] for _ in range(3)]
 
         # PITCH & DRUM PITCHES & BAR & POSITIONS & PROGRAM
-        vocab[0] += [f"Pitch_{i}" for i in self.pitch_range]
-        vocab[0] += [f"DrumPitch_{i}" for i in self.drum_pitch_range]
+        vocab[0] += [f"Pitch_{i}" for i in range(*self.config.pitch_range)]
+        vocab[0] += [
+            f"DrumPitch_{i}"
+            for i in range(*self.config.additional_params["drum_pitch_range"])
+        ]
         vocab[0] += ["Bar_None"]  # new bar token
-        nb_positions = max(self.beat_res.values()) * 4  # 4/* time signature
+        max_nb_beats = max(
+            map(lambda ts: ceil(4 * ts[0] / ts[1]), self.time_signatures)
+        )
+        nb_positions = max(self.config.beat_res.values()) * max_nb_beats
         vocab[0] += [f"Position_{i}" for i in range(nb_positions)]
-        vocab[0] += [f"Program_{program}" for program in self.programs]
+        vocab[0] += [f"Program_{program}" for program in self.config.programs]
 
         # BAR POS ENC
-        vocab[1] += [f"BarPosEnc_{i}" for i in range(self.max_bar_embedding)]
+        vocab[1] += [
+            f"BarPosEnc_{i}"
+            for i in range(self.config.additional_params["max_bar_embedding"])
+        ]
 
         # POSITION POS ENC
         vocab[2] += [
@@ -487,15 +420,15 @@ class MuMIDI(MIDITokenizer):
         vocab[2] += [f"PositionPosEnc_{i}" for i in range(nb_positions)]  # pos enc
 
         # CHORD
-        if self.additional_tokens["Chord"]:
+        if self.config.use_chords:
             vocab[0] += self._create_chords_tokens()
 
         # REST
-        if self.additional_tokens["Rest"]:
+        if self.config.use_rests:
             vocab[0] += [f'Rest_{".".join(map(str, rest))}' for rest in self.rests]
 
         # TEMPO
-        if self.additional_tokens["Tempo"]:
+        if self.config.use_tempos:
             vocab.append([f"Tempo_{i}" for i in self.tempos])
 
         # Velocity and Duration in last position
@@ -525,7 +458,7 @@ class MuMIDI(MIDITokenizer):
         dic["Pitch"] = ["Pitch", "Program", "Bar", "Position"]
         dic["DrumPitch"] = ["DrumPitch", "Program", "Bar", "Position"]
 
-        if self.additional_tokens["Chord"]:
+        if self.config.use_chords:
             dic["Program"] += ["Chord"]
             dic["Chord"] = ["Pitch"]
 
