@@ -2,6 +2,7 @@ from .utils import (
     convert_ids_tensors_to_list,
     get_midi_programs,
     remove_duplicated_notes,
+    fix_offsets_overlapping_notes,
     detect_chords,
     merge_tracks_per_class,
     merge_tracks,
@@ -13,6 +14,7 @@ __all__ = [
     "convert_ids_tensors_to_list",
     "get_midi_programs",
     "remove_duplicated_notes",
+    "fix_offsets_overlapping_notes",
     "detect_chords",
     "merge_tracks_per_class",
     "merge_tracks",
